# GemRB - Infinity Engine Emulator
# Copyright (C) 2007 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

#Note: keep this file in sync with PCStatStructs.h
ACT_STEALTH=0
ACT_THIEVING=1
ACT_CAST=2
ACT_QSPELL1=3
ACT_QSPELL2=4
ACT_QSPELL3=5
ACT_TURN=6
ACT_TALK=7
ACT_USE=8
ACT_QSLOT1=9
ACT_QSLOT4=10 #heh
ACT_QSLOT2=11
ACT_QSLOT3=12
ACT_QSLOT5=31 #
ACT_INNATE=13
ACT_DEFEND=14 
ACT_ATTACK=15
ACT_WEAPON1=16
ACT_WEAPON2=17
ACT_WEAPON3=18
ACT_WEAPON4=19
ACT_BARDSONG=20
ACT_STOP=21
ACT_SEARCH=22
ACT_SHAPE=23
ACT_TAMING=24
ACT_SKILLS=25
ACT_WILDERNESS=26
ACT_DANCE = 27
#gui navigation (scrolling button rows left or right)
ACT_LEFT=32
ACT_RIGHT=33

ACT_NONE=100

ACT_BARD = 40
ACT_CLERIC = 41
ACT_DRUID = 42
ACT_PALADIN = 43
ACT_RANGER = 44
ACT_SORCERER = 45
ACT_WIZARD = 46
ACT_DOMAIN = 47
ACT_WILDSHAPE = 48

ACT_IWDQSPELL = 50
ACT_IWDQITEM = 60
ACT_IWDQSPEC = 70
ACT_IWDQSONG = 80
