# GemRB - Infinity Engine Emulator
# Copyright (C) 2011 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# a listing of iwd2 feat indices

# KEEP SYNCHRONISED WITH gemrb/includes/ie_feats.h !!!
FEAT_AEGIS_OF_RIME = 0
FEAT_AMBIDEXTERITY = 1
FEAT_AQUA_MORTIS = 2
FEAT_ARMOUR_PROFICIENCY = 3
FEAT_ARMORED_ARCANA = 4
FEAT_ARTERIAL_STRIKE = 5
FEAT_BLIND_FIGHT = 6
FEAT_BULLHEADED = 7
FEAT_CLEAVE = 8
FEAT_COMBAT_CASTING = 9
FEAT_COURTEOUS_MAGOCRACY = 10
FEAT_CRIPPLING_STRIKE = 11
FEAT_DASH = 12
FEAT_DEFLECT_ARROWS = 13
FEAT_DIRTY_FIGHTING = 14
FEAT_DISCIPLINE = 15
FEAT_DODGE = 16
FEAT_ENVENOM_WEAPON = 17
FEAT_EXOTIC_BASTARD = 18
FEAT_EXPERTISE = 19
FEAT_EXTRA_RAGE = 20
FEAT_EXTRA_SHAPESHIFTING = 21
FEAT_EXTRA_SMITING = 22
FEAT_EXTRA_TURNING = 23
FEAT_FIENDSLAYER = 24
FEAT_FORESTER = 25
FEAT_GREAT_FORTITUDE = 26
FEAT_HAMSTRING = 27
FEAT_HERETICS_BANE = 28
FEAT_HEROIC_INSPIRATION = 29
FEAT_IMPROVED_CRITICAL = 30
FEAT_IMPROVED_EVASION = 31
FEAT_IMPROVED_INITIATIVE = 32
FEAT_IMPROVED_TURNING = 33
FEAT_IRON_WILL = 34
FEAT_LIGHTNING_REFLEXES = 35
FEAT_LINGERING_SONG = 36
FEAT_LUCK_OF_HEROES = 37
FEAT_MARTIAL_AXE = 38
FEAT_MARTIAL_BOW = 39
FEAT_MARTIAL_FLAIL = 40
FEAT_MARTIAL_GREATSWORD = 41
FEAT_MARTIAL_HAMMER = 42
FEAT_MARTIAL_LARGESWORD = 43
FEAT_MARTIAL_POLEARM = 44
FEAT_MAXIMIZED_ATTACKS = 45
FEAT_MERCANTILE_BACKGROUND = 46
FEAT_POWER_ATTACK = 47
FEAT_PRECISE_SHOT = 48
FEAT_RAPID_SHOT = 49
FEAT_RESIST_POISON = 50
FEAT_SCION_OF_STORMS = 51
FEAT_SHIELD_PROF = 52
FEAT_SIMPLE_CROSSBOW = 53
FEAT_SIMPLE_MACE = 54
FEAT_SIMPLE_MISSILE = 55
FEAT_SIMPLE_QUARTERSTAFF = 56
FEAT_SIMPLE_SMALLBLADE = 57
FEAT_SLIPPERY_MIND = 58
FEAT_SNAKE_BLOOD = 59
FEAT_SPELL_FOCUS_ENCHANTMENT = 60
FEAT_SPELL_FOCUS_EVOCATION = 61
FEAT_SPELL_FOCUS_NECROMANCY = 62
FEAT_SPELL_FOCUS_TRANSMUTE = 63
FEAT_SPELL_PENETRATION = 64
FEAT_SPIRIT_OF_FLAME = 65
FEAT_STRONG_BACK = 66
FEAT_STUNNING_FIST = 67
FEAT_SUBVOCAL_CASTING = 68
FEAT_TOUGHNESS = 69
FEAT_TWO_WEAPON_FIGHTING = 70
FEAT_WEAPON_FINESSE = 71
FEAT_WILDSHAPE_BOAR = 72
FEAT_WILDSHAPE_PANTHER = 73
FEAT_WILDSHAPE_SHAMBLER = 74
