#predefined sound indices

DEF_OPEN = 0
DEF_CLOSE = 1
DEF_HOPEN = 2  
DEF_HCLOSE  = 3
DEF_BUTTON1 = 4
DEF_BUTTON2 = 5
DEF_BUTTON3 = 6
DEF_OPENFAIL = 7
DEF_CLOSEFAIL = 8
DEF_ITEM_GONE = 9
DEF_SECRET = 10
DEF_PICKLOCK = 11
#reserved 12-24
DEF_SOLD = 25
DEF_STOLEN = 26
DEF_DRUNK = 27
DEF_DONATE1 = 28
DEF_DONATE2 = 29
DEF_IDENTIFY = 30
DEF_GOTXP = 31

# sfx channel names from sndchann.2da; keep synced with Audio::Audio in core
CHAN_NARRATOR = "NARRATIO"
CHAN_AMB_AREA = "AREA_AMB"
CHAN_ACTIONS = "ACTIONS"
CHAN_SWINGS = "SWINGS"
CHAN_CASTING = "CASTING"
CHAN_GUI = "GUI"
CHAN_DIALOG = "DIALOG"
CHAN_CHAR1 = "CHARACT0"
CHAN_CHAR2 = "CHARACT1"
CHAN_CHAR3 = "CHARACT2"
CHAN_CHAR4 = "CHARACT3"
CHAN_CHAR5 = "CHARACT4"
CHAN_CHAR6 = "CHARACT5"
CHAN_CHAR7 = "CHARACT6"
CHAN_CHAR8 = "CHARACT7"
CHAN_CHAR9 = "CHARACT8"
CHAN_CHAR10 = "CHARACT9"
CHAN_MONSTER = "MONSTER"
CHAN_HITS = "HITS"
CHAN_MISSILE = "MISSILE"
CHAN_AMB_LOOP = "AMBIENTL"
CHAN_AMB_NONL = "AMBIENTN"
CHAN_WALK_CHAR = "WALKINGC"
CHAN_WALK_MONSTER = "WALKINGM"
CHAN_ARMOR = "ARMOR"

# sound handling flags, from Audio.h
SND_SPATIAL = 1
SND_LOOPING = 2
SND_SPEECH = 4
SND_QUEUE = 8
SND_EFX = 16

# from Audio.h
GEM_SND_VOL_MUSIC = 1
GEM_SND_VOL_AMBIENTS = 2
