# GemRB - Infinity Engine Emulator
# Copyright (C) 2006 The GemRB Project
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

# ie_restype.py - definitions of engine resource types

# !!! NOTE: Keep this file synchronized with gemrb/includes/SClassID.h !!!

RES_2DA			=  0x000003F4
RES_ACM			=  0x00010000
RES_ARE			=  0x000003F2
RES_BAM			=  0x000003E8
RES_BCS			=  0x000003EF
RES_BS			=  0x100003EF
RES_BIF			=  0x00020000
RES_BMP			=  0x00000001
RES_PNG			=  0x00000003
RES_CHR			=  0x000003FA
RES_CHU			=  0x000003EA
RES_CRE			=  0x000003F1
RES_DLG			=  0x000003F3
RES_EFF			=  0x000003F8
RES_GAM			=  0x000003F5
RES_IDS			=  0x000003F0
RES_INI			=  0x00000802
RES_ITM			=  0x000003ED
RES_KEY			=  0x00030000
RES_MOS			=  0x000003EC
RES_MUS			=  0x00040000
RES_MVE			=  0x00000002
RES_PLT			=  0x00000006
RES_PRO			=  0x000003FD
RES_SAV			=  0x00050000
RES_SPL			=  0x000003EE
RES_SRC			=  0x00000803
RES_STO			=  0x000003F6
RES_TIS			=  0x000003EB
RES_TLK			=  0x00060000
RES_TOH			=  0x00070000
RES_TOT			=  0x00080000
RES_VAR			=  0x00090000
RES_VVC			=  0x000003FB
RES_WAV			=  0x00000004
RES_WED			=  0x000003E9
RES_WFX			=  0x00000005
RES_WMP			=  0x000003F7
RES_VIDEO		=  0x000B0000
RES_AUDIO		=  0x000C0000
RES_SCRIPT		=  0x000D0000
RES_GUI_SCRIPT		=  0x000E0000
RES_COMPRESSION 	=  0x000F0000
RES_FX			=  0x00100000
